@@ -10,6 +10,7 @@ from pprint import pformat
 from types import ModuleType
 from typing import (
     Any,
+    AnyStr,
     Callable,
     Dict,
     Generic,
@@ -29,12 +30,12 @@ from typing import (
 
 from jsonbourne import jsonlib
 
-JsonPrimitiveT = TypeVar('JsonPrimitiveT', str, int, float, None)
-JsonObjT = TypeVar('JsonObjT', bound='JsonObj')
-KT = TypeVar('KT')
-VT = TypeVar('VT')
+JsonPrimitiveT = TypeVar("JsonPrimitiveT", str, int, float, None)
+JsonObjT = TypeVar("JsonObjT", bound="JsonObj")
+KT = TypeVar("KT")
+VT = TypeVar("VT")
 _KT = str
-_VT = TypeVar('_VT')
+_VT = TypeVar("_VT")
 
 if sys.version_info < (3, 7):
     from collections.abc import MutableMapping
@@ -46,25 +47,25 @@ else:
     JsonObjMutableMapping = MutableMapping[str, _VT]
 
 __all__ = (
-    'JsonObj',
-    'JsonDict',
-    'JsonObjMutableMapping',
-    'stringify',
-    'parse',
-    'jsonify',
-    'JSON',
-    'UNDEFINED',
-    'undefined',
-    'Null',
-    'null',
-    'JSONModuleCls',
-    'JsonObjT',
+    "JsonObj",
+    "JsonDict",
+    "JsonObjMutableMapping",
+    "stringify",
+    "parse",
+    "jsonify",
+    "JSON",
+    "UNDEFINED",
+    "undefined",
+    "Null",
+    "null",
+    "JSONModuleCls",
+    "JsonObjT",
 )
 
 _JsonObjMutableMapping_attrs = set(dir(JsonObjMutableMapping))
 
-UNDEFINED = 'undefined'
-undefined = 'undefined'
+UNDEFINED = "undefined"
+undefined = "undefined"
 null = None
 Null = None.__class__
 
@@ -121,7 +122,7 @@ def is_int(value: Any) -> bool:
     if isinstance(value, int):
         return True
     _value: str = str(value)
-    if _value[0] in ('-', '+'):
+    if _value[0] in ("-", "+"):
         return str(_value[1:]).isdigit()
     return _value.isdigit()
 
@@ -205,13 +206,13 @@ class JsonObj(JsonObjMutableMapping, Generic[_VT]):
     def __init__(self, *args: Any, **kwargs: Any) -> None:
         """Use the object dict"""
         _data = dict(*args, **kwargs)
-        super().__setattr__('_data', _data)
+        super().__setattr__("_data", _data)
         try:
             assert all(isinstance(k, str) for k in self._data)
         except AssertionError:
             d = {k: v for k, v in self._data.items() if not isinstance(k, str)}
             raise ValueError(
-                'JsonObj keys MUST be strings! Bad key values: {}'.format(str(d))
+                "JsonObj keys MUST be strings! Bad key values: {}".format(str(d))
             )
         self.recurse()
         self.__post_init__()
@@ -231,7 +232,7 @@ class JsonObj(JsonObjMutableMapping, Generic[_VT]):
         """Check if a key or dot-key is contained within the JsonObj object
 
         Args:
-            key (str): root level key or a dot-key
+            key (_KT): root level key or a dot-key
 
         Returns:
             bool: True if the key/dot-key is in the JsonObj; False otherwise
@@ -251,8 +252,8 @@ class JsonObj(JsonObjMutableMapping, Generic[_VT]):
             False
 
         """
-        if '.' in key:
-            first_key, _, rest = key.partition('.')
+        if "." in key:
+            first_key, _, rest = key.partition(".")
             val = self._data.get(first_key)
             return isinstance(val, MutableMapping) and val.__contains__(rest)
         return key in self._data
@@ -269,7 +270,7 @@ class JsonObj(JsonObjMutableMapping, Generic[_VT]):
         """Set JsonObj item with 'key' to 'value'
 
         Args:
-            key (str): Key/item to set
+            key (_KT): Key/item to set
             value: Value to set
 
         Returns:
@@ -297,7 +298,7 @@ class JsonObj(JsonObjMutableMapping, Generic[_VT]):
             return None
         if not is_identifier(key):
             raise ValueError(
-                f'Invalid key: ({key}).\n' f'Key(s) is not a valid python identifier'
+                f"Invalid key: ({key}).\n" f"Key(s) is not a valid python identifier"
             )
         self._data[key] = value
 
@@ -329,9 +330,9 @@ class JsonObj(JsonObjMutableMapping, Generic[_VT]):
             2
 
         """
-        if item == '_data':
+        if item == "_data":
             try:
-                return object.__getattribute__(self, '_data')
+                return object.__getattribute__(self, "_data")
             except AttributeError:
                 return self.__dict__
         if item in _JsonObjMutableMapping_attrs or item in self._cls_protected_attrs():
@@ -406,7 +407,7 @@ class JsonObj(JsonObjMutableMapping, Generic[_VT]):
                 return default
             raise ke
 
-    def filter_none(self, recursive: bool = False) -> 'JsonObj':
+    def filter_none(self, recursive: bool = False) -> "JsonObj":
         """Filter key-values where the value is `None` but not false-y
 
         Args:
@@ -491,7 +492,7 @@ class JsonObj(JsonObjMutableMapping, Generic[_VT]):
             )
         return JsonObj({k: v for k, v in self.items() if v is not None})
 
-    def filter_false(self, recursive: bool = False) -> 'JsonObj':
+    def filter_false(self, recursive: bool = False) -> "JsonObj":
         """Filter key-values where the value is false-y
 
         Args:
@@ -625,31 +626,32 @@ class JsonObj(JsonObjMutableMapping, Generic[_VT]):
 
         Raises:
             KeyError: Raised if the dot-key is not in in the object
+            ValueError: Raised if key is not a str/Tuple[str, ...]/List[str]
 
         """
         if not isinstance(key, (str, list, tuple)):
             raise ValueError(
-                ''.join(
+                "".join(
                     (
-                        'dot_key arg must be string or sequence of strings; ',
+                        "dot_key arg must be string or sequence of strings; ",
                         "strings will be split on '.'",
                     )
                 )
             )
-        parts = key.split('.') if isinstance(key, str) else list(key)
+        parts = key.split(".") if isinstance(key, str) else list(key)
         root_val: Any = self._data[parts[0]]
         cur_val = root_val
         for ix, part in enumerate(parts[1:], start=1):
             try:
                 cur_val = cur_val[part]
             except TypeError:
-                reached = '.'.join(parts[:ix])
-                err_msg = f'Invalid DotKey: {key} -- Lookup reached: {reached} => {str(cur_val)}'
+                reached = ".".join(parts[:ix])
+                err_msg = f"Invalid DotKey: {key} -- Lookup reached: {reached} => {str(cur_val)}"
                 if isinstance(key, str):
-                    err_msg += ''.join(
+                    err_msg += "".join(
                         (
                             f"\nNOTE!!! lookup performed with string ('{key}') ",
-                            'PREFER lookup using List[str] or Tuple[str, ...]',
+                            "PREFER lookup using List[str] or Tuple[str, ...]",
                         )
                     )
                 raise KeyError(err_msg)
@@ -698,15 +700,15 @@ class JsonObj(JsonObjMutableMapping, Generic[_VT]):
     def to_str(self, minify: bool = False, width: int = 88) -> str:
         """Return a string representation of the JsonObj object"""
         if minify:
-            return type(self).__name__ + '(**' + str(self.to_dict()) + ')'
-        return ''.join(
+            return type(self).__name__ + "(**" + str(self.to_dict()) + ")"
+        return "".join(
             [
                 type(self).__name__,
-                '(**{\n    ',
-                pformat(self.to_dict(), width=width)[1:-1].replace('\n', '\n   '),
-                '\n})',
+                "(**{\n    ",
+                pformat(self.to_dict(), width=width)[1:-1].replace("\n", "\n   "),
+                "\n})",
             ]
-        ).replace('JsonObj(**{}),', '{},')
+        ).replace("JsonObj(**{}),", "{},")
 
     def __repr__(self) -> str:
         """Return the string representation of the object"""
@@ -718,7 +720,7 @@ class JsonObj(JsonObjMutableMapping, Generic[_VT]):
 
     def _repr_html_(self) -> str:
         """Return the HTML representation of the JsonObj object"""
-        return '<pre>{}</pre>'.format(self.__str__())
+        return "<pre>{}</pre>".format(self.__str__())
 
     @classmethod
     def _cls_attr_names(cls) -> Set[str]:
@@ -726,7 +728,7 @@ class JsonObj(JsonObjMutableMapping, Generic[_VT]):
         try:
             return {el.name for el in cls.__attrs_attrs__}  # type: ignore
         except AttributeError:
-            raise AttributeError('Class is not decorated with attr.attrs')
+            raise AttributeError("Class is not decorated with attr.attrs")
 
     @classmethod
     def _cls_fields(cls) -> Set[str]:
@@ -734,7 +736,7 @@ class JsonObj(JsonObjMutableMapping, Generic[_VT]):
         try:
             return cls.__fields__  # type: ignore
         except AttributeError:
-            raise AttributeError('Class does not inherit from pydantic.BaseModel')
+            raise AttributeError("Class does not inherit from pydantic.BaseModel")
 
     @classmethod
     def _cls_field_names(cls) -> Set[str]:
@@ -742,49 +744,49 @@ class JsonObj(JsonObjMutableMapping, Generic[_VT]):
         try:
             return set(cls.__fields__)  # type: ignore
         except AttributeError:
-            raise AttributeError('Class does not inherit from pydantic.BaseModel')
+            raise AttributeError("Class does not inherit from pydantic.BaseModel")
 
     @classmethod
     def _cls_protected_attrs(cls) -> Set[str]:
         """Return attrs-attribute names for an object decorated with attrs"""
         return {
-            'asdict',
-            'clear',
-            'copy',
-            'dot_items',
-            'dot_items_list',
-            'dot_keys',
-            'dot_keys_list',
-            'dot_keys_set',
-            'dot_lookup',
-            'eject',
-            'entries',
-            'filter_false',
-            'filter_none',
-            'from_dict',
-            'from_json',
-            'fromkeys',
-            'get',
-            'items',
-            'keys',
-            'pop',
-            'popitem',
-            'recurse',
-            'setdefault',
-            'stringify',
-            'to_dict',
-            'to_json',
-            'to_str',
-            'update',
-            'validate_type',
-            'values',
+            "asdict",
+            "clear",
+            "copy",
+            "dot_items",
+            "dot_items_list",
+            "dot_keys",
+            "dot_keys_list",
+            "dot_keys_set",
+            "dot_lookup",
+            "eject",
+            "entries",
+            "filter_false",
+            "filter_none",
+            "from_dict",
+            "from_json",
+            "fromkeys",
+            "get",
+            "items",
+            "keys",
+            "pop",
+            "popitem",
+            "recurse",
+            "setdefault",
+            "stringify",
+            "to_dict",
+            "to_json",
+            "to_str",
+            "update",
+            "validate_type",
+            "values",
         }
 
     def _field_names(self) -> Set[str]:
         """Return attrs-attribute names for an object decorated with attrs"""
         return self.__class__._cls_field_names()
 
-    def eject(self) -> Dict[_KT, Any]:
+    def eject(self) -> Dict[_KT, _VT]:
         """Eject to python-builtin dictionary object
 
         Examples:
@@ -802,7 +804,7 @@ class JsonObj(JsonObjMutableMapping, Generic[_VT]):
             return {k: unjsonify(v) for k, v in self._data.items()}
         except RecursionError:
             raise ValueError(
-                'JSON.stringify recursion err; cycle/circular-refs detected'
+                "JSON.stringify recursion err; cycle/circular-refs detected"
             )
 
     def to_dict(self) -> Dict[_KT, Any]:
@@ -819,81 +821,160 @@ class JsonObj(JsonObjMutableMapping, Generic[_VT]):
         return cls(**data)
 
     @classmethod
-    def from_json(cls: Type[JsonObjT], json_string: str) -> JsonObjT:
+    def from_json(cls: Type[JsonObjT], json_string: AnyStr) -> JsonObjT:
         """Return a JsonObj object from a json string
 
         Args:
             json_string (str): JSON string to convert to a JsonObj
 
         Returns:
-            JsonObj: JsonObj object for the given JSON string
+            JsonObjT: JsonObj object for the given JSON string
 
         """
         return cls._from_json(json_string)
 
     @classmethod
-    def _from_json(cls: Type[JsonObjT], json_string: str) -> JsonObjT:
+    def _from_json(cls: Type[JsonObjT], json_string: AnyStr) -> JsonObjT:
         """Return a JsonObj object from a json string
 
         Args:
             json_string (str): JSON string to convert to a JsonObj
 
         Returns:
-            JsonObj: JsonObj object for the given JSON string
+            JsonObjT: JsonObj object for the given JSON string
 
         """
         return cls.from_dict(jsonlib.loads(json_string))
 
-    def to_json(
-        self, pretty: bool = False, sort_keys: bool = False, **kwargs: Any
+    def JSON(
+        self,
+        fmt: bool = False,
+        pretty: bool = False,
+        sort_keys: bool = False,
+        append_newline: bool = False,
+        default: Optional[Callable[[Any], Any]] = None,
+        **kwargs: Any,
     ) -> str:
-        """Return a JSON string of the JsonObj object
+        """Return JSON string of the JsonObj object (and children)
 
         Args:
-            minify (bool): Return a 'minified' version of the JSON string
-            sort_keys (bool): Sort the keys when converting to JSON
-            indent (int): Indent level of the json string
-            **kwargs: Keyword args to be passed on to the JSON dumps method
-
-        Returns:
-            str: JSON string of the JsonObj object
-
-        """
-        return self._to_json(pretty=pretty, sort_keys=sort_keys, **kwargs)
-
-    def stringify(
-        self, pretty: bool = False, sort_keys: bool = False, **kwargs: Any
-    ) -> str:
-        """Return a JSON string of the JsonObj; `JsonObj.to_json` alias
-
-        Args:
-            pretty (bool): If a 'minified' version of the JSON string
-            sort_keys (bool): Sort the keys when converting to JSON
-            **kwargs: Keyword args to be passed on to the JSON dumps method
-
-        Returns:
-            str: JSON string of the JsonObj object
-
-        """
-        return self.to_json(pretty=pretty, sort_keys=sort_keys, **kwargs)
-
-    def _to_json(
-        self, pretty: bool = False, sort_keys: bool = False, **kwargs: Any
-    ) -> str:
-        """Return a JSON string of the JsonObj object
-
-        Args:
-            minify (bool): Return a 'minified' version of the JSON string
-            sort_keys (bool): Sort the keys when converting to JSON
-            indent (int): Indent level of the json string
-            **kwargs: Keyword args to be passed on to the JSON dumps method
+            fmt (bool): If True, return a JSON string with newlines and indentation
+            pretty (bool): If True, return a JSON string with newlines and indentation
+            sort_keys (bool): Sort dictionary keys if True
+            append_newline (bool): Append a newline '\n' to JSON string if True
+            default: default function hook for JSON serialization
+            **kwargs (Any): additional kwargs to be passed down to jsonlib.dumps
 
         Returns:
             str: JSON string of the JsonObj object
 
         """
         return jsonlib.dumps(
-            self.to_dict(), pretty=pretty, sort_keys=sort_keys, **kwargs
+            self.to_dict(),
+            fmt=fmt,
+            pretty=pretty,
+            sort_keys=sort_keys,
+            append_newline=append_newline,
+            default=default,
+            **kwargs,
+        )
+
+    def to_json(
+        self,
+        fmt: bool = False,
+        pretty: bool = False,
+        sort_keys: bool = False,
+        append_newline: bool = False,
+        default: Optional[Callable[[Any], Any]] = None,
+        **kwargs: Any,
+    ) -> str:
+        """Return JSON string of the JsonObj object (and children)
+
+        Args:
+            fmt (bool): If True, return a JSON string with newlines and indentation
+            pretty (bool): If True, return a JSON string with newlines and indentation
+            sort_keys (bool): Sort dictionary keys if True
+            append_newline (bool): Append a newline '\n' to JSON string if True
+            default: default function hook for JSON serialization
+            **kwargs (Any): additional kwargs to be passed down to jsonlib.dumps
+
+        Returns:
+            str: JSON string of the JsonObj object
+
+        """
+        return self._to_json(
+            fmt=fmt,
+            pretty=pretty,
+            sort_keys=sort_keys,
+            append_newline=append_newline,
+            default=default,
+            **kwargs,
+        )
+
+    def stringify(
+        self,
+        fmt: bool = False,
+        pretty: bool = False,
+        sort_keys: bool = False,
+        append_newline: bool = False,
+        default: Optional[Callable[[Any], Any]] = None,
+        **kwargs: Any,
+    ) -> str:
+        """Return JSON string of the JsonObj object (and children)
+
+        Args:
+            fmt (bool): If True, return a JSON string with newlines and indentation
+            pretty (bool): If True, return a JSON string with newlines and indentation
+            sort_keys (bool): Sort dictionary keys if True
+            append_newline (bool): Append a newline '\n' to JSON string if True
+            default: default function hook for JSON serialization
+            **kwargs (Any): additional kwargs to be passed down to jsonlib.dumps
+
+        Returns:
+            str: JSON string of the JsonObj object
+
+        """
+        return self._to_json(
+            fmt=fmt,
+            pretty=pretty,
+            sort_keys=sort_keys,
+            append_newline=append_newline,
+            default=default,
+            **kwargs,
+        )
+
+    def _to_json(
+        self,
+        fmt: bool = False,
+        pretty: bool = False,
+        sort_keys: bool = False,
+        append_newline: bool = False,
+        default: Optional[Callable[[Any], Any]] = None,
+        **kwargs: Any,
+    ) -> str:
+        """Return JSON string of the JsonObj object (and children)
+
+        Args:
+            fmt (bool): If True, return a JSON string with newlines and indentation
+            pretty (bool): If True, return a JSON string with newlines and indentation
+            sort_keys (bool): Sort dictionary keys if True
+            append_newline (bool): Append a newline '\n' to JSON string if True
+            default: default function hook for JSON serialization
+            **kwargs (Any): additional kwargs to be passed down to jsonlib.dumps
+
+        Returns:
+            str: JSON string of the JsonObj object
+
+        """
+
+        return jsonlib.dumps(
+            self.to_dict(),
+            fmt=fmt,
+            pretty=pretty,
+            sort_keys=sort_keys,
+            append_newline=append_newline,
+            default=default,
+            **kwargs,
         )
 
     @classmethod
@@ -913,11 +994,6 @@ class JsonDict(JsonObj):
     pass
 
 
-# @overload
-# def jsonify(value: Dict[_KT, _VT]) -> JsonObj[_VT]:
-#     ...
-
-
 def as_json_obj(value: Union[JsonObj[_VT], Dict[_KT, _VT]]) -> JsonObj[_VT]:
     if isinstance(value, dict):
         return JsonObj(value)
@@ -997,8 +1073,10 @@ class JSON(metaclass=JSONMeta):
     @staticmethod
     def stringify(
         data: Any,
+        fmt: bool = False,
         pretty: bool = False,
         sort_keys: bool = False,
+        append_newline: bool = False,
         default: Optional[Callable[[Any], Any]] = None,
         **kwargs: Any,
     ) -> str:
@@ -1006,8 +1084,10 @@ class JSON(metaclass=JSONMeta):
         return str(
             jsonlib.dumps(
                 data,
+                fmt=fmt,
                 pretty=pretty,
                 sort_keys=sort_keys,
+                append_newline=append_newline,
                 default=default,
                 **kwargs,
             )
@@ -1016,8 +1096,10 @@ class JSON(metaclass=JSONMeta):
     @staticmethod
     def dumps(
         data: Any,
+        fmt: bool = False,
         pretty: bool = False,
         sort_keys: bool = False,
+        append_newline: bool = False,
         default: Optional[Callable[[Any], Any]] = None,
         **kwargs: Any,
     ) -> str:
@@ -1025,8 +1107,10 @@ class JSON(metaclass=JSONMeta):
         return str(
             jsonlib.dumps(
                 data,
+                fmt=fmt,
                 pretty=pretty,
                 sort_keys=sort_keys,
+                append_newline=append_newline,
                 default=default,
                 **kwargs,
             )
@@ -1035,8 +1119,10 @@ class JSON(metaclass=JSONMeta):
     @staticmethod
     def binify(
         data: Any,
+        fmt: bool = False,
         pretty: bool = False,
         sort_keys: bool = False,
+        append_newline: bool = False,
         default: Optional[Callable[[Any], Any]] = None,
         **kwargs: Any,
     ) -> bytes:
@@ -1044,8 +1130,10 @@ class JSON(metaclass=JSONMeta):
         return bytes(
             jsonlib.dumpb(
                 data,
+                fmt=fmt,
                 pretty=pretty,
                 sort_keys=sort_keys,
+                append_newline=append_newline,
                 default=default,
                 **kwargs,
             )
@@ -1054,8 +1142,10 @@ class JSON(metaclass=JSONMeta):
     @staticmethod
     def dumpb(
         data: Any,
+        fmt: bool = False,
         pretty: bool = False,
         sort_keys: bool = False,
+        append_newline: bool = False,
         default: Optional[Callable[[Any], Any]] = None,
         **kwargs: Any,
     ) -> bytes:
@@ -1063,22 +1153,44 @@ class JSON(metaclass=JSONMeta):
         return bytes(
             jsonlib.dumpb(
                 data,
+                fmt=fmt,
                 pretty=pretty,
                 sort_keys=sort_keys,
+                append_newline=append_newline,
                 default=default,
                 **kwargs,
             )
         )
 
     @staticmethod
-    def loads(string: str, obj: bool = False, **kwargs: Any) -> Any:
+    def jsoncp(
+        data: Any,
+        fmt: bool = False,
+        pretty: bool = False,
+        sort_keys: bool = False,
+        append_newline: bool = False,
+        default: Optional[Callable[[Any], Any]] = None,
+        **kwargs: Any,
+    ) -> Any:
+        return jsonlib.jsoncp(
+            data=data,
+            fmt=fmt,
+            pretty=pretty,
+            sort_keys=sort_keys,
+            append_newline=append_newline,
+            default=default,
+            **kwargs,
+        )
+
+    @staticmethod
+    def loads(string: AnyStr, obj: bool = False, **kwargs: Any) -> Any:
         """Parse JSON string/bytes and return raw representation"""
         if obj:
             return jsonify(jsonlib.loads(string, **kwargs))
         return jsonlib.loads(string, **kwargs)
 
     @staticmethod
-    def parse(string: str, obj: bool = True) -> Any:
+    def parse(string: AnyStr, obj: bool = True) -> Any:
         """Parse JSON string/bytes and jsonify all dictionaries -> JsonObj"""
         if obj:
             return jsonify(jsonlib.loads(string))
@@ -1135,7 +1247,7 @@ def _cls_protected_attrs(cls: Any) -> Set[str]:
 stringify = JSON.stringify
 parse = JSON.parse
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     import doctest
 
     doctest.testmod()
